/* Title: Button Messages
   Name: Eric Walsh
	Description: This file contains the messages and creation options for a drag
					and drop	button.

		Messages:
			DB_LEFT:   Left align any supplied text.
						  DB_CENTER: Center align any supplied text (default).
			DB_RIGHT:  Right align any supplied text.
			DB_OWNERDRAW: Puts in a supplied bitmap/ with any text.
			DB_3STATE: ON/OFF type button.
						  DB_WANTMOUSEMOVE: Sends any mouse move messages to the parent.
*/

#ifndef BUTTON_H
#define BUTTON_H

#define DB_LEFT             0x00000000L
#define DB_RIGHT            0x00000001L
#define DB_OWNERDRAW        0x00000002L
#define DB_SWITCHBUTTON     0x00000004L
#define DB_WANTMOUSEMOVE    0x00000008L
#define DB_STATIC	    		 0x00000010L

/* DB_LEFT           -Left Aligns text
	DB_RIGHT          -Right Aligns text
	DB_OWNERDRAW      -Allows for bitmaps to be drawn on button
	DB_SWITCHBUTTON   -Like a switch, click it, it stays down
	DB_WANTMOUSEMOVE  -Sends back mouse move messages over button back to parent
	DB_STATIC         -Not implemented yet
*/

/*************************** Messages to Control ****************/

#define DB_SETBITMAPS       (WM_USER +0x0010)
#define DB_GETCHECK         (WM_USER +0x0011)
#define DB_SETCHECK         (WM_USER +0x0012)
#define DB_SETTRANSPARENT   (WM_USER +0x0013)

/************************** Messages from Control ***************/

#define DB_MOVE		       (WM_USER +0x0100)

#define DB_DRAGNDROP        0x10
#define DB_BUTTONSEL        0x11

/***************************************************************************/
/* The following is a list of all programmer sendable messages to the      */
/* drag and drop button.                                                   */
/***************************************************************************/

/*

Important Note: Since this control is defined (as far as windows is concerned)
as a button there is some default button messages sent by the system to this
control.  Those messages are from WM_USER+1->WM_USER+4.  All messages defined
for this control must be out of that range to avoid conflicts.

--------------------------------------------------------------------------------

DB_SETBITMAPS    -Supplies bitmaps for DB_OWNERDRAW (owner drawn) buttons.
	wParam       -->Number of Bitmaps being supplied
	lParam       -->Array of bitmaps handles, number set by wParam.

	RETURN VALUE: Number of bitmaps set.

--------------------------------------------------------------------------------

DB_GETCHECK      -Gets state of DB_SWITCHBUTTON buttons
	wParam       -->0
	lParam       -->0L

	RETURN VALUE: 0 button is up. 1 button is down.

--------------------------------------------------------------------------------

DB_SETCHECK      -Sets state of DB_SWITCHBUTTON buttons
	wParam       -->0, set the button to up.  1, set the button to down.
	lParam       -->0L

	RETURN VALUE: Returns TRUE if successful, FALSE if not

--------------------------------------------------------------------------------

DB_SETTRANSPARENT -Sets the bitmap color to make transparent.  Allow bitmap to
						 blend in with the button face to matter the color.  Only for
						 use with DB_OWNERDRAW buttons.
	wParam       -->0
	lParam       -->RGB COLORREF to make transparent

	RETURN VALUE: Returns TRUE if successful, FALSE if not

*/

#endif
