/* Title: Drag and Drop Header File
	Name: Eric Walsh
	Description: This file contains all the user sendable and notification
					 message for drag and drop.  This file is to be used for any
                control that uses this implemenation of drag and drop.
*/

#ifndef _DRAGDROP_H_
#define _DRAGDROP_H_ 

// File Manager Server Drag & Drop Internal Structure. (UnDocumented)

#define DD_ERR -1

#define DD_DROPSELECTED       (WM_USER +0x1000)
#define DD_VALIDDROPCURSOR    (WM_USER +0x1001)
#define DD_INVALIDDROPCURSOR  (WM_USER +0x1002)
#define DD_DRAGNDROPWIN	      (WM_USER +0x1003)
#define DD_SETDROPCURSOR      (WM_USER +0x1004)
#define DD_MOUSEMOVE          (WM_USER +0x1005)
#define DD_MOUSEMOVEEXIT      (WM_USER +0x1006)

#define DD_INITFILEBUFFER     (WM_USER +0x1007)
#define DD_INITFILEAPPEND     (WM_USER +0x1008)
#define DD_GETFILEBUFFER      (WM_USER +0x1009)

#define MN_SETMENU            (WM_USER +0x1010)
#define MN_MENUSTART          (WM_USER +0x1011)

/****** Below are the Messages that can be sent to a drag and drop windows *****

--------------------------------------------------------------------------------

DD_DROPSELECTED    -Informs the control dropped over that it has been dropped
						  over.
	wParam       -->0
	lParam       -->0L

	RETURN VALUE: TRUE

--------------------------------------------------------------------------------

DD_VALIDDROPCURSOR -Specifies the cursor to use when over a valid
						  WM_EX_ACCEPTFILES control.  This message (cursor) can
						  be overridden with the WM_SETDROPCURSOR.
	wParam       -->cursor to use for valid drag and drop controls.
	lParam       -->0L

	RETURN VALUE: TRUE

--------------------------------------------------------------------------------

DD_INVALIDDROPCURSOR -Specifies the cursor to use when over a invalid
						  (not WM_EX_ACCEPTFILES) control.  This message (cursor) can
						  be overridden with the WM_SETDROPCURSOR.
	wParam       -->cursor to use
	lParam       -->0L

	RETURN VALUE: TRUE

--------------------------------------------------------------------------------

DD_DRAGNDROPWIN    -Retrieves the Window Handle of the Window the Drag and Drop
						  cursor is over
	wParam       -->0
   lParam       -->0L

	RETURN VALUE: Handle of Window Drag and Drop Cursor is over

--------------------------------------------------------------------------------

DD_SETDROPCURSOR   -Allows cursors sent by means of DD_VALIDDROPCURSOR or
						  DD_INVALIDDROPCURSOR to be overridden with a new cursor.
						  Setting this cursor to zero will clear the cursor and return
						  previous valid and invalid cursors during drag and drop.
	wParam       -->cursor
	lParam       -->0L

	RETURN VALUE: TRUE

--------------------------------------------------------------------------------

DD_MOUSEMOVE   -Sent to a WS_EXACCEPTFILES window when a drag and drop cursor is
					 over the window.  This call operates like WM_MOUSEMOVE
	wParam       -->Key state of various keys (see WM_MOUSEMOVE)
	lParam       -->LOWORD is xpos, HIWORD is ypos of cursor

	RETURN VALUE: TRUE

--------------------------------------------------------------------------------

DD_MOUSEMOVEEXIT -Sent to a WS_EXACCEPTFILES window when a drag and drop cursor
						is leaving the window.  This call acts like WM_MOUSEMOVE
						otherwise.
	wParam       -->Key state of various keys (see WM_MOUSEMOVE)
	lParam       -->LOWORD is xpos, HIWORD is ypos of cursor

	RETURN VALUE: TRUE

--------------------------------------------------------------------------------

DD_INITFILEBUFFER - Send to Initialize File Manager Drag & Drop Server Support

   wParam       -->0;
	lParam       -->0;

	RETURN VALUE: TRUE if successful, FALSE if failure.

--------------------------------------------------------------------------------

DD_INITFILEAPPEND - Send with a file Name to append to the list of Drag & drop
                    Files.  Must Call DD_INITFILEBUFFER Before Sending Message.

   wParam       -->0;
	lParam       -->Pointer to ASCIIZ File Name.

	RETURN VALUE: TRUE if successful, FALSE if failure.

*/

/*********** Below are the drag and drop notification messages ****************/

#define DDN_STARTDRAGNDROP    0x1000
#define DDN_DONEDRAGNDROP     0x1001
#define DDN_DRAGNDROPNEWWIN   0x1002

/******************** List Box Type Notification Messages ***************

DDN_STARTDRAGNDROP -Sent when drag and drop has been started.
	wParam       --> Specifies the identifier of the editable listbox
	lParam       --> Specifies handle of editable listbox in the low word,
						  and specifies DDN_STARTDRAGNDROP in the high word.

--------------------------------------------------------------------------------

DDN_DONEDRAGNDROP  -Sent when drag and drop has been completed (button up).
	wParam       --> Specifies the identifier of the editable listbox
	lParam       --> Specifies handle of editable listbox in the low word,
						  and specifies DDN_DONEDRAGNDROP in the high word.

--------------------------------------------------------------------------------

DDN_DRAGNDROPNEWWIN-Signals the drag and drop cursor has moved over a new window
	wParam       --> Specifies the identifier of the editable listbox
	lParam       --> Specifies handle of editable listbox in the low word,
						  and specifies DDN_DRAGNDROPNEWWIN in the high word.

	Return Value: The user should return a handle to a cursor.  If no cursor
					  (FALSE) is returned the cursor supplied with a appropriate
					  DD_VALIDDROPCURSOR/ DD_INVALIDDROPCURSOR message will be used.
					  If they are both invalid the cursor will not change.  A valid
					  window is figured by using the WM_EX_ACCEPTFILES flag.


--------------------------------------------------------------------------------

*******************************************************************************/

/* === UNDOCUMENTED MESSAGES === */

#define WM_DROPOBJECT			0x022A
#define WM_QUERYDROPOBJECT		0x022B

#define WM_BEGINDRAG			   0x022C
#define WM_DRAGLOOP				0x022D

#define WM_DRAGSELECT			0x022E
#define WM_DRAGMOVE				0x022F

#ifndef WM_DROPFILES
#define WM_DROPFILES		    	0x0233
#endif


/* === UNDOCUMENTED CONSTANTS === */

// Used in the wFlags field of the DRAGINFO sructure

#define DRAGOBJ_PROGRAM			0x0001	// A single executable
#define DRAGOBJ_DATA			   0x0002	// A single 'other' file
#define DRAGOBJ_DIRECTORY		0x0003	// A single pathed directory
#define DRAGOBJ_MULTIPLE		0x0004	// Any combination 2 or more
#define DRAGOBJ_EXTERNAL		0x8000	// ORed with one of the above

// Used as the return to the source (caller of DragObject)

#define DRAG_PRNT 0x544e5250
#define DRAG_FILE 0x454c4946

/* === UNDOCUMENTED STRUCTURE === */

// Structure pointed at by lParam of WM_DRAGDROP
typedef struct {
	HWND hwndSource;	// File Manager window that owns source listbox
	HANDLE h1;			// Handle of drag icon ???
	WORD wFlags;		// One of the DRAGOBJ_ constants below. They
   						// include a top bit set, indicating drag
	   					// is allowed outside source app (FileMan).
	char *szList;		// near pointer to list of files.
	HANDLE hOfstruct;	// handle to a global OFSTRUCT. If a list, or
		   				// file has no association, FileMan sets
			   			// this to NULL.
	int	x, y;			// Position of cursor at drop
	long lUnknown;		// ????
	} DRAGINFO, FAR * LPDRAGINFO;


// Structure in global memory - handle in wParam of WM_DROPFILES in 3.1

typedef struct
   {
   WORD  wSize;               // Size of data structure
   POINT ptMousePos;          // Position of mouse cursor
   BOOL  fInNonClientArea;    // Was the mouse in the window's non-client area
   } DROPFILESTRUCT, FAR *LPDROPFILESTRUCT;

#endif

