/* Title: Drive Utitlities File
   Name: Eric Walsh
   Description: This file contains the main files for the drive windows.
                This control will read in and display the number and type
                of each drive, associating the appropriate bitmap with the
                drive.  The control will create scroll buttons, and scroll
                when appropriate.  The drive bank can be either multiple or
                single select.

		  File:
		          drvcntl() -main constructor drive bank. Reads in drive bitmaps
                           and drive types and creates a button to represent it.
               ~drvcntl() -main desctructor, deletes all bitmaps;
                WinSpec() -sets size of control, display only entire buttons it
                           control and buts scroll arrows up to see remainder.
         ProcessButtons() -Will scroll left and right if needed, inform parent of
                           drive selection.
           GetNumDrives() -Returns number of drives to parent.
          GetDriveState() -Gets drive state.
          SetDriveState() -Sets drive state to be selected or unselected.
     GetNumActiveDrives() -Gets number of selected drives.

*/

#include <windows.h>
#include <string.h>
#include <bios.h>
#include "button.h"
#include "drvclass.h"
#include "drive.h"

/****************************************************************************/
/* drvcntl(HWND)                                                            */
/*                                                                          */
/* Arguments:                                                               */
/*    HWND -Handle to Editable Listbox Contrl                               */
/*                                                                          */
/* drvcntl() is the constructor for the drive bank.  The routine reads in   */
/* the drive types, the drive bitmaps and creates a button to represent     */
/* the drive.                                                               */
/****************************************************************************/

drvcntl::drvcntl(HWND hwnd)

{
	int      cnt, type;
	char     drive[256];
	BITMAP   bm;
	union    REGS inregs;
	long     style;
	HBITMAP  hBitmap;
	RECT     rect;
	COLORREF tColor;

	hWnd =hwnd;
	ActiveWnd =0;
	ButLeft =ButRght =0;
	vscrlindx =0;
	VertNodesScrn =0;
	fl35bitmaps =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "Flop35");
	fl525bitmaps =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "Flop525");
	HDbitmaps =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "HardDrive");
	Netbitmaps =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "NetDrive");
	Leftbitmap =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "LeftButt");
	Rghtbitmap =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "RightButt");

	NumDrives =0;
	for (cnt =0; cnt < 26; cnt++)  // Loop through all drives;
	{
		if (GetDriveType(cnt) !=0)
			NumDrives++;
	}
	ButWnd =new HWND[NumDrives];

	NumDrives =0;
	strcpy(drive, "X:");
	BarLength =0;
	style =WS_CHILD | WS_VISIBLE | WS_BORDER | DB_OWNERDRAW | DB_RIGHT | DB_SWITCHBUTTON;

	for (cnt =0; cnt < 26; cnt++)  // Loop through all drives;
	{
	   tColor =RGB(0, 0, 255);
		if ((type =GetDriveType(cnt)) !=0)
		{
			drive[0] =cnt +'A';

			switch (type)
			{
				case DRIVE_REMOVABLE:
					inregs.h.ah =0x8;
					inregs.h.dl =cnt;
					int86(0x13, &inregs, &inregs);

					if (inregs.h.bl > 2)
					{
						hBitmap =fl35bitmaps;
						tColor =RGB(255, 0, 0);
					}
					else
						hBitmap =fl525bitmaps;
					break;

				case DRIVE_FIXED:
					hBitmap =HDbitmaps;
					break;

				case DRIVE_REMOTE:
					hBitmap =Netbitmaps;
					break;
			}

			GetObject(hBitmap, sizeof(BITMAP), (LPSTR) &bm);
			bm.bmHeight+=9;
			bm.bmWidth+=9;
			ButWnd[NumDrives] =CreateWindowEx (WS_EX_ACCEPTFILES, "DnDButton", drive, style,
					bm.bmWidth* NumDrives, 0, bm.bmWidth, bm.bmHeight, hWnd, cnt,
					GetWindowWord(hWnd, GWW_HINSTANCE), NULL);
			if (ButWnd[NumDrives])
			{
				SendMessage(ButWnd[NumDrives], DB_SETBITMAPS, 1, (LONG)&hBitmap);
				SendMessage(ButWnd[NumDrives], DB_SETTRANSPARENT, 0, (LONG)tColor);
				ShowWindow(ButWnd[NumDrives], SW_SHOWNORMAL);
				GetClientRect(ButWnd[NumDrives], &rect);
				BarLength +=rect.right;
			}
			else
				return ;

			NumDrives++;
		}
	}
}

/****************************************************************************/
/* ~drvcntl()                                                               */
/*                                                                          */
/* ~drvcntl() is the desctructor for the drive bank and deletes all drive   */
/* bitmaps.                                                                 */
/****************************************************************************/

drvcntl::~drvcntl()

{
	DeleteObject(fl35bitmaps);
	DeleteObject(fl525bitmaps);
	DeleteObject(HDbitmaps);
	DeleteObject(Netbitmaps);
	DeleteObject(Leftbitmap);
	DeleteObject(Rghtbitmap);
}

/****************************************************************************/
/* WinSpec()                                                                */
/*                                                                          */
/* WinSpec() is the resize routine for the drive bank. This routine         */
/* positions the buttons within the control and creates the left and right  */
/* scroll buttons if necessary. This routine will destroy the scroll buttons*/
/* if they are no longer needed.                                            */
/*                                                                          */
/* Return Value: TRUE if successful, false if not.                          */
/****************************************************************************/


LONG drvcntl::WinSpec()

{
	RECT rect;
	BITMAP bm;
	int  cnt;

	GetClientRect(hWnd, &ScrlRect);
	if (BarLength > ScrlRect.right)
	{
		GetObject(Leftbitmap, sizeof(BITMAP), (LPSTR) &bm);
		bm.bmWidth +=9;
		bm.bmHeight +=9;
		ScrlRect.left +=bm.bmWidth;
		ScrlRect.right -=bm.bmWidth;

		if (!IsWindow(ButLeft))
		{
			ButLeft =CreateWindowEx (WS_EX_ACCEPTFILES, "DnDButton", "",
						WS_CHILD | WS_VISIBLE | WS_BORDER | DB_OWNERDRAW | WS_DISABLED,
						0, 0, bm.bmWidth, bm.bmHeight, hWnd, IDD_BUTTONLEFT,
						GetWindowWord(hWnd, GWW_HINSTANCE), NULL);
         if (!ButLeft)
				return FALSE;
			SendMessage(ButLeft, DB_SETBITMAPS, 1, (LONG)&Rghtbitmap);

			ButRght =CreateWindowEx (WS_EX_ACCEPTFILES, "DnDButton", "",
						WS_CHILD | WS_VISIBLE | WS_BORDER | DB_OWNERDRAW,
						ScrlRect.right, 0, bm.bmWidth, bm.bmHeight, hWnd, IDD_BUTTONRIGHT,
						GetWindowWord(hWnd, GWW_HINSTANCE), NULL);
			if (!ButRght)
            return FALSE;
			SendMessage(ButRght, DB_SETBITMAPS, 1, (LONG)&Leftbitmap);
		}
		else
			SetWindowPos(ButRght, NULL, ScrlRect.right, 0, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE);

		xpos =ScrlRect.left;
		VertNodesScrn =0;
		for (cnt =0; cnt <NumDrives; cnt++)
		{
			GetClientRect(ButWnd[cnt], &rect);

			if (xpos +rect.right  < ScrlRect.right)
			{
				SetWindowPos(ButWnd[cnt], HWND_BOTTOM, xpos -1, 0, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE);
				xpos +=(rect.right +1);
				VertNodesScrn++;
				ShowWindow(ButWnd[cnt], SW_NORMAL);
				EnableWindow(ButWnd[cnt], 1);
			}
			else
			{
				ShowWindow(ButWnd[cnt], SW_HIDE);
				EnableWindow(ButWnd[cnt], 0);
			}
		}
	}
	else
	if (ButLeft)
	{
		DestroyWindow(ButLeft);
		DestroyWindow(ButRght);
		ButLeft =ButRght =0;

		xpos =0;
		for (cnt =0; cnt <NumDrives; cnt++)
		{
			GetClientRect(ButWnd[cnt], &rect);

			if (xpos +rect.right < ScrlRect.left + ScrlRect.right)
			{
				SetWindowPos(ButWnd[cnt], HWND_BOTTOM, xpos, 0, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE);
				xpos +=(rect.right +1);
				ShowWindow(ButWnd[cnt], SW_SHOWNORMAL);
				EnableWindow(ButWnd[cnt], 1);
			}
		}
	}

	xpos -=(rect.right +1);
	return TRUE;
}

/****************************************************************************/
/* ProcessButtons()                                                         */
/*                                                                          */
/* ProcessButtons() will process all button, including the scroll buttons   */
/* if the scroll buttons are pressed the appropriate scroll action will     */
/* occur.  If a drive button is pressed a message is sent the parent.       */
/****************************************************************************/

LONG drvcntl::ProcessButtons(WORD wParam, LONG lParam)

{
	int check;
	RECT lrect;

	GetClientRect(ButLeft, &lrect);
	if (wParam ==IDD_BUTTONRIGHT)     // Advance (show) later buttons
	{
		RECT rect;

		int pvscrlindx =vscrlindx;

		vscrlindx =min(vscrlindx +1, NumDrives -VertNodesScrn);
		if (pvscrlindx !=vscrlindx)
		{
			if (vscrlindx >=1)
				EnableWindow(ButLeft, TRUE);
			if (vscrlindx ==NumDrives -VertNodesScrn)
			  EnableWindow(ButRght, FALSE);
			ShowWindow(ButWnd[vscrlindx -1], SW_HIDE);
			EnableWindow(ButWnd[vscrlindx -1], 0);
			
			GetClientRect(ButWnd[vscrlindx], &rect);
			ScrollWindowEx(hWnd, -rect.right -1, 0, &ScrlRect, &ScrlRect, NULL, NULL, SW_ERASE | SW_INVALIDATE | SW_SCROLLCHILDREN);

			SetWindowPos(ButWnd[vscrlindx +VertNodesScrn -1], NULL, xpos, 0, 0, 0, SWP_NOSIZE);
			ShowWindow(ButWnd[vscrlindx +VertNodesScrn -1], SW_SHOWNORMAL);
			EnableWindow(ButWnd[vscrlindx +VertNodesScrn -1], 1);
		}
	}
	else
	if (wParam ==IDD_BUTTONLEFT)               // show earlier buttons
	{
		RECT rect;

		int pvscrlindx =vscrlindx;
		vscrlindx =max(0, vscrlindx -1);

		if (pvscrlindx !=vscrlindx)
		{
			if (vscrlindx ==0)
				EnableWindow(ButLeft, FALSE);
			if (vscrlindx <NumDrives -VertNodesScrn)
			  EnableWindow(ButRght, TRUE);
			ShowWindow(ButWnd[vscrlindx +VertNodesScrn], SW_HIDE);
			EnableWindow(ButWnd[vscrlindx +VertNodesScrn], 0);
			GetClientRect(ButWnd[vscrlindx], &rect);
			ScrollWindowEx(hWnd, rect.right +1, 0, &ScrlRect, &ScrlRect, NULL, NULL, SW_ERASE | SW_INVALIDATE | SW_SCROLLCHILDREN);

			SetWindowPos(ButWnd[vscrlindx], NULL, lrect.right +1, 0, 0, 0, SWP_NOSIZE);
			ShowWindow(ButWnd[vscrlindx], SW_SHOWNORMAL);
			EnableWindow(ButWnd[vscrlindx], 1);
		}
	}
	else
	{
      if (ActiveWnd ==LOWORD(lParam))
      {
	      if ((GetWindowLong(hWnd, GWL_STYLE) & DRS_CONSTANTSELECT) ==0)
   	      SendMessage(ActiveWnd, DB_SETCHECK, 1, 0L);
         SendMessage(GetParent(hWnd), WM_COMMAND, GetWindowWord(hWnd, GWW_ID), MAKELONG(hWnd, DRN_DRIVERESELECTED));
      }
      else
	   if ((GetWindowLong(hWnd, GWL_STYLE) & DRS_MULTIPLESEL) ==0)
   	{
			/* This section sets the previously depressed button back up */

			check =(int)SendMessage(ActiveWnd, DB_GETCHECK, 0, 0L);
			if (ActiveWnd && check && ActiveWnd != LOWORD(lParam))
				SendMessage(ActiveWnd, DB_SETCHECK, 0, 0L);

			ActiveWnd =LOWORD(lParam);
			SendMessage(GetParent(hWnd), WM_COMMAND, GetWindowWord(hWnd, GWW_ID), MAKELONG(hWnd, DRN_DRIVECHANGE));
		}
	}
	return TRUE;
}

/****************************************************************************/
/* GetDriveState()                                                          */
/*                                                                          */
/* GetDriveState() returns the state of the specified drive. This routine   */
/* only works with multiple select drive banks.                             */
/*                                                                          */
/* Return Value: TRUE if button is down, FALSE if up.                       */
/****************************************************************************/

LONG drvcntl::GetDriveState(WORD drv)

{
	HWND dWnd;

	dWnd =GetDlgItem(hWnd, drv);
	if (drv >NumDrives || !IsWindow(dWnd))
		return DR_ERR;
	if ((GetWindowLong(hWnd, GWL_STYLE) & DRS_MULTIPLESEL) ==0)
		return DR_ERR;

	return SendMessage(dWnd, DB_GETCHECK, 0, 0L);
}

/****************************************************************************/
/* SetDriveState()                                                          */
/*                                                                          */
/* SetDriveState() sets the state of the specified drive. This routine      */
/* only works with multiple select drive banks.                             */
/*                                                                          */
/* Return Value: FALSE.                                                     */
/****************************************************************************/

LONG drvcntl::SetDriveState(WORD state, LONG drv)

{
	HWND dWnd;

	dWnd =GetDlgItem(hWnd, (int)drv);
	if (drv >NumDrives || !IsWindow(dWnd))
		return DR_ERR;
	if ((GetWindowLong(hWnd, GWL_STYLE) & DRS_MULTIPLESEL) ==0)
		return DR_ERR;

	return SendMessage(dWnd, DB_SETCHECK, state, 0L);
}

/****************************************************************************/
/* GetNumActiveDrives()                                                     */
/*                                                                          */
/* GetNumActiveDrives() retruns the number of selected drives.  This routine*/
/* oly works with multiple select drive banks.                              */                                           
/*                                                                          */
/* Return Value: Number of active drives.                                   */
/****************************************************************************/

LONG drvcntl::GetNumActiveDrives()

{
	register int cnt, NumActive;
	HWND dWnd;

	if ((GetWindowLong(hWnd, GWL_STYLE) & DRS_MULTIPLESEL) ==0)
		return DR_ERR;

	NumActive =0;
	for (cnt =0; cnt <NumDrives; cnt++)
	{
		dWnd =GetDlgItem(hWnd, cnt);
		if (IsWindow(dWnd))
		{
			if (SendMessage(hWnd, DB_GETCHECK, 0, 0L))
				NumActive++;
		}
	}

	return NumActive;
}
