/* Title: Hierarchy User messages
	Name: Eric Walsh
	Description: This file contains the creation, message and notify message
					 defines for the hierarchy control.
*/

#ifndef _HRCHY_H
#define _HRCHY_H_

#define HC_ERR -1

/************** Hierarchy Control Creation Style Parameters ******************/

#define HCS_MULTIPLESEL     0x00000001L
#define HCS_DRAGDROPSERVER  0x00000002L
#define HCS_MULTIPLEHRCHY   0x00000004L
#define HCS_SORT            0x00000008L
#define HCS_DIRECTORYTREE   0x00000010L

/*
HCS_MULTIPLESEL    -Allows the Hierarchy to have mulitple selection for the
						  items in the hierarchy.
HCS_DRAGDROPSERVER -Takes the hierarchy control a drag and drop server.
HCS_MULTIPLEHRCHY  -Allows the hierarchy to have mulitple trees associated with
						  a window, giving the control the ability to switch trees
						  in and out of view.
HCS_SORT				 -Sorts the hierarchy in alphabetical order (done by level).						  
HCS_DIRECTORYTREE  -Sets up a hierarchy to display a specified drive.  This
						  style automatically sets HCS_MULTIPLEHRCHY
/************** User Sendable Hierarchy Control Messages *********************/

#define HC_GETCURSEL             (WM_USER +0x0001)
#define HC_SETCURSEL             (WM_USER +0x0002)
#define HC_GETCARETINDEX         (WM_USER +0x0003)
#define HC_ADDSTRING             (WM_USER +0x0004)
#define HC_DELETESTRING          (WM_USER +0x0005)
#define HC_GETTEXT               (WM_USER +0x0006)
#define HC_GETFULLPATHTEXT       (WM_USER +0x0007)
#define HC_GETTEXTLEN            (WM_USER +0x0008)
#define HC_GETCOUNT              (WM_USER +0x0009)
#define HC_GETTOTALCOUNT         (WM_USER +0x000A)
#define HC_FINDSTRING            (WM_USER +0x000B)
#define HC_SETITEMEXPANDEDSTATE  (WM_USER +0x000C)
#define HC_GETITEMEXPANDEDSTATE  (WM_USER +0x000D)
#define HC_SETSEL                (WM_USER +0x000E)
#define HC_GETSEL                (WM_USER +0x000F)
#define HC_GETITEMDATA           (WM_USER +0x0010)
#define HC_SETITEMDATA           (WM_USER +0x0011)
#define HC_SETDELIMITERCHAR      (WM_USER +0x0012)
#define HC_CREATEHRCHY			   (WM_USER +0x0013)
#define HC_GETDISPLAYHRCHY		   (WM_USER +0x0014)
#define HC_SETDISPLAYHRCHY		   (WM_USER +0x0015)
#define HC_SETMEMORYACCESORHRCHY (WM_USER +0x0016)
#define HC_SETITEMBITMAP         (WM_USER +0x0017)
#define HC_GETITEMBITMAP         (WM_USER +0x0018)
#define HC_SETBITMAPTRANSCOLOR   (WM_USER +0x0019)
#define HC_SETACCESSHRCHYMEM     (WM_USER +0x001A)
#define HC_GETMOUSEDROPINDEX     (WM_USER +0x001B)

/************* User Sendable Directory Hierarchy Messages *********************/

#define HC_SELECTDRIVE       		(WM_USER +0x0101)
#define HC_GETNUMDIRECS      		(WM_USER +0x0103)
#define HC_SETPATHDIRECSTATE 		(WM_USER +0x0109)


/*
IMPORTANT NOTE: When a user collapses a branch using this hierarchy the children
					 below the item and any states (specifically when multiple
					 select is on) will remain if memory and redisplayed when the
					 branch is re-expanded.  Hidden (collapsed) items can be accessed
					 by using HC_SETACCESSHRCHYMEM

--------------------------------------------------------------------------------

HC_GETCURSEL       -Gets the currently selected item in the hierarchy
	wParam        -->0
	lParam        -->0L

	Return Value: Returns index of selected item if one exists, HC_ERR if not.
					  Use HC_GETCARETINDEX to get the currently selected item in
					  a multiple selection hierarchy

--------------------------------------------------------------------------------

HC_SETCURSEL       -Sets the currently selected item in the hierarchy
	wParam        -->Index
	lParam        -->0L

	Return Value: Returns 0 on success, HC_ERR if not.

--------------------------------------------------------------------------------

HC_GETCARETINDEX    -Gets the item with the focus in a multiple selection
							hierarchy control.

	wParam        -->0
	lParam        -->0L

	Return Value: Returns index of selected item if one exists, HC_ERR if not.

--------------------------------------------------------------------------------

HC_ADDSTRING       -Adds an item (string) to the parent node specified, creating
						  a child below the parent.  If the HCS_SORT creation style is
						  set the child will be added in alpabetical order with other
						  sibling directly below the parent.  If HCS_SORT is not set
						  the string will be added to the end of the children.

	wParam        -->Parent (Sequential node index) to add the child item
						 (string) to.
	lParam        -->Pointer to string

	Return Value: Returns TRUE if able to addstring HC_ERR if not.

--------------------------------------------------------------------------------

HC_DELETESTRING    -Deletes a string (child) at the node specified. All child
						  nodes associated with the item are also deleted.

	wParam        -->Index of string to delete
	lParam        -->Flag indicating whether to delete the parent (TRUE will
						  delete the parent).

	Return Value: Returns TRUE if able to delete the item, FALSE if not.

--------------------------------------------------------------------------------

HC_GETTEXT         -Gets the text of the specified index.

	wParam        -->Index of string to retrieve
	lParam        -->Pointer to buffer to insert text into.

	Return Value: Returns TRUE if able to retrieve the text

--------------------------------------------------------------------------------

HC_GETFULLPATHTEXT -Gets the full text of the specified index.  The full text is
						  the specified index plus all parents, grand parents etc.
						  The order in the string is oldest parent down to the index
						  specified.  The paths are delimited by the character
						  supplied by the HC_SETDELIMIIERCHAR. A leading delimiter is
						  supplied but no trailing delimiter is. 

	wParam        -->Index of string to retrieve
	lParam        -->Pointer to buffer to insert text into.

	Return Value: Returns TRUE if able to retrieve the text

--------------------------------------------------------------------------------

HC_GETTEXTLEN      -Get the text length of the specified index or full path as
						  specified.

	wParam        -->Index of string to get text length of.
	lParam        -->If FALSE retrives the text length of index only.
						  If TRUE retrives the length of entire path plus delimiter.

	Return Value: Length of string.

--------------------------------------------------------------------------------

HC_GETCOUNT       -Gets the number of visible items on the control beginning at
						  the position specified and giving all children

	wParam        -->Starting index (-1 gives count for entire control).
	lParam        -->Flag stating whether to count only children or all 
	                 descendents. FALSE will count all descendents.

	Return Value: Returns number of visibible items

--------------------------------------------------------------------------------

HC_GETTOTALCOUNT   -Gets the total number of items (visible and hidden)
						  belonging to the control beginning at the position specified
						  and giving all children

	wParam        -->Starting index (-1 gives count for entire control).
	lParam        -->0L

	Return Value: Returns number of total items

--------------------------------------------------------------------------------

HC_FINDSTRING      -Searchs for the specified string in the hierarchy.

	wParam        -->Starting index (-1 gives count for entire control).
	lParam        -->String to Search for.

Note: If the search string has delimiters the search routine will find the first
		full path string that matches the entire search string.  In other words
		if the search string has one delimiter the search will search every item,
		loooking for a match then match the parent.

	Return Value: Return the child most node that matches the string.  HC_ERR is
					  returned if the search was unsuccesful. Index is zero based.

--------------------------------------------------------------------------------

HC_SETITEMEXPANDEDSTATE -Sets the expanded state of the specified item and all
							  descendents

	wParam        -->Sets Expand state (FALSE is collapse, TRUE expand)
	lParam        -->Item Index

	Return Value: TRUE if able to set item state, HC_ERR if not.

--------------------------------------------------------------------------------

HC_GETITEMEXPANDEDSTATE -Return whether an item is visible or hidden

	wParam        -->Index
	lParam        -->OL

	Return Value: TRUE if item is expanded, FALSE if not, HC_ERR is returned if
					  an error occurs.

--------------------------------------------------------------------------------

HC_SETSEL               -Sets the selection state of an item

	wParam        -->Selection State (TRUE selects the item).
	lParam        -->Item Index

	Return Value: TRUE if succesful, HC_ERR if not

	Note: Only for use with Mulitple select hierarchy controls

--------------------------------------------------------------------------------

HC_GETSEL               -Gets the selection state of an item

	wParam        -->Item Index
	lParam        -->0L

	Return Value: TRUE if selected, FALSE if not, HC_ERR is returned if an error
					  occured.

	Note: Only for use with Mulitple select hierarchy controls

--------------------------------------------------------------------------------

HC_GETITEMDATA          -Gets the Item data associated with the index.

	wParam        -->Item Index
	lParam        -->0L

	Return Value: Return the 32 bit value associated with the item.

--------------------------------------------------------------------------------

HC_SETITEMDATA          -Sets the Item data associated with the index.

	wParam        -->Item Index
	lParam        -->Value to associate with item

	Return Value: TRUE if succesful, HC_ERR if not

--------------------------------------------------------------------------------

HC_SETITEMBITMAP	-Sets the bitmap associated with a certain item.  The bitmap
						 will be displayed before the text associated with the item.
						 The item state can be gotten using HC_GETITEMEXPANDEDSTATE.
						 HCN_BITMAPSELCHANGE will be sent when the user clicks on
						 the displayed bitmap. To remove any bitmap send a bitmap
						 handle of zero.

	wParam        -->Item Index
	lParam        -->Bitmap Handle

	Return Value: TRUE if bitmap is set, HC_ERR is returned if an error occurs.

--------------------------------------------------------------------------------

HC_GETITEMBITMAP  -Gets the bitmap associated with an item.

	wParam        -->Item Index
	lParam        -->0L

   Return Value: HBITMAP if a bitmap is assigned, 0 if not.

--------------------------------------------------------------------------------

HC_SETBITMAPTRANSCOLOR -Sets the color to make transparent when displaying
								bitmaps.  This allows different colored backgrounds to
								display the bitmap without seeing a background. This
								color will be used throughout the hierarchy for all
								bitmaps. Setting the color to -1 will clear the
								transparent color.

	wParam        -->O
	lParam        -->RGB color to make transparent.

	Return Value: No Return Value

--------------------------------------------------------------------------------

HC_SETDELIMITERCHAR -Sets the character to delimit the text concatanated by
							HC_GETFULLTEXTPATH.  Setting the character to zero removes
							any delimiter character.  The defualt is '.' (period)

	wParam        -->Character to delimit string
	lParam        -->0L

	Return Value: No return code.

--------------------------------------------------------------------------------

HC_CREATEHRCHY     -For use with HCS_MULITPLEHRCHY. Informs the control to
						  create a hierarchy with id set in wParam. If the
						  HCS_MULTIPLEHRCHY flag is not set one hierarchy will be
						  created and calls to this message will return HC_ERR.

	wParam        -->Flag stating whether to display the hierarchy (TRUE is
						  display).
	lParam        -->Id to associate with new hierarchy

	Return Value: No return code.

	Note: If wParam is TRUE the active display hierarchy is set to this
	      hierarchy, the memory hierarchy is automatically set to this hierarchy. 

--------------------------------------------------------------------------------

HC_GETDISPLAYHRCHY -For use with HCS_MULITPLEHRCHY. Returns the id of which
							control is currently being displayed.

	wParam        -->0
	lParam        -->0L

	Return Value: Which hierarchy is currently being displayed. 

--------------------------------------------------------------------------------

HC_SETDISPLAYHRCHY -For use with HCS_MULITPLEHRCHY. Informs the control which
						  hierarchy to display.  Must be created using the
						  HC_CREATEHRCHY message.

	wParam        -->Flag stating whether to make this hierarchy the memory
	                 hierarchy. (TRUE is make memory also)
	lParam        -->Id of hierarchy to display

	Return Value: TRUE if set to hierarchy specified by lParan, HC_ERR if
	              an error occurred.

--------------------------------------------------------------------------------

HC_SETMEMORYACCESORHRCHY -For use with HCS_MULITPLEHRCHY. Allows the hierarchy's
						  memory to be accessed even though the control is not
						  displayed.  All calls such as HC_ADDSTRING and HC_GETSEL
						  will operate on the hierarchy specified and not the
						  displayed hierarchy.

	wParam        -->Id of hierarchy to display
	lParam        -->0L

	Return Value: TRUE if set to hierarchy set by the tree, FALSE if an error.

--------------------------------------------------------------------------------

HC_SETACCESSHRCHYMEM  -Sets a flag indicating whether accessor function
								 HC_ADDSTRING, HC_INSERTSTRING, HC_DELETESTRING,
								 HC_GETTEXT, HC_GETFULLPATHTEXT, HC_FINDSTRING,
								 HC_GETITEMDATA, HC_SETITEMDATA, HC_SETITEMBITMAP are
								 referencing visible or all items.  Visible items are
								 the items viewable through scrolling.  All items are
								 visible items plus any items under a collapsed branch.

								 Setting all items is useful if the hierarchy is
								 multiple and the programmer wish to get the state
								 of collapsed branches.

								 Use HC_GETCOUNT and HC_GETTOTALCOUNT to see if nodes
								 are collapsed.

								 The default is accessing only visible nodes.

	wParam			-->TRUE acesses visible memory, FALSE will access all memory
	lParam			-->OL

	Return Value: No return value:

--------------------------------------------------------------------------------

HC_GETMOUSEDROPINDEX -Gets the index where the drop operation, in a drag and
							 drop operation occurred.  If the cursor was not over a
							 valid item -1 will be returned.  This could possibly mean
							 to add a node to the highest level.

	wParam         -->0
	lParam		   -->0L

	Return Value: Returns a zero based drop index, HC_ERR is returned if none. 

--------------------------------------------------------------------------------

************************* Directory Hierarchy messages *************************

--------------------------------------------------------------------------------

HC_SELECTDRIVE  -Reads in and displays the specified drive.

	wParam        -->Drive to read and display (0 =A, 1 =B, 2=C etc.)
	lParam        -->0L

	Return Value: Returns TRUE if a valid drive and a successful display.
					  TW_ERR is returned if an error occurs.

--------------------------------------------------------------------------------

HC_GETNUMDIRECS    -Gets the number of directories (and subdirectories)
						  in the drive specified.
	wParam        -->Drive to to retrieve (0 =A, 1 =B, 2=C etc.)
	lParam        -->0L

	Return Value: Returns number of directories in the specified drive.
					  TW_ERR is returned if an error occurs.

--------------------------------------------------------------------------------

HC_SETPATHDIRECSTATE -Sets the state of the specified directory path entry. This
						 message only works with multiple selection trees.  If the
						 path is not loaded, it will expand the tree to load the full
						 path.

	wParam         -->state to set to directory to.  TRUE is set FALSE is unset
	lParam         -->char * to the Directory Path

	Return Value: Returns TRUE if set correctly. TW_ERR is returned if an
					  error occurs.

--------------------------------------------------------------------------------

*/

/************** Notify messages from the Hierarchy Control ********************/

#define HCN_SELCHANGE     		1
#define HCN_DBLCLK        		2
#define HCN_SETFOCUS      		3
#define HCN_KILLFOCUS     		4
#define HCN_BITMAPSELCHANGE 	5
#define HCN_BITMAPDBLCLK      6
#define HCN_ERRSPACE          7

/*
--------------------------------------------------------------------------------

HCN_SELCHANGE    -Sent when a user selects a new item with the mouse of when
						the Enter key is hit.

	wParam       --> Specifies the identifier of the hierarchy control
	lParam       --> Specifies handle of hierarchy control in the low word,
						  and specifies HCN_SELCHANGE in the high word.

--------------------------------------------------------------------------------

HCN_DBLCLK       -Sent when a double clicks on an entry.  This message is
						usally associated with activating the item (aka lauching
						a file).

	wParam       --> Specifies the identifier of the hierarchy control
	lParam       --> Specifies handle of hierarchy control in the low word,
						  and specifies HCN_DBLCLK in the high word.

--------------------------------------------------------------------------------

HCN_SETFOCUS     -Sent when the hierarchy gains focus

	wParam       --> Specifies the identifier of the hierarchy control
	lParam       --> Specifies handle of hierarchy control in the low word,
						  and specifies HCN_SETFOCUS in the high word.

--------------------------------------------------------------------------------

HCN_KILLFOCUS    -Sent when the hierarchy kills focus

	wParam       --> Specifies the identifier of the hierarchy control
	lParam       --> Specifies handle of hierarchy control in the low word,
						  and specifies HCN_KILLFOCUS in the high word.

--------------------------------------------------------------------------------

HCN_BITMAPSELCHANGE -Sent when the item bitmap is selected with a mouse or the
							space key is hit.  This message is normally associated
							with expanding and collapsing branches. Return FALSE to
							have the branches expand and collapse automatically.

	wParam       --> Specifies the identifier of the hierarchy control
	lParam       --> Specifies handle of hierarchy control in the low word,
						  and specifies HCN_BITMAPSELCHANGE in the high word.

--------------------------------------------------------------------------------

HCN_BITMAPDBLCLK   -Sent when the item bitmap is double clicked with a mouse or
							the enter key is hit.  This message is normally associated
							with activating an item or expanding and collapsing
							branches if part of a directory tree.

	wParam       --> Specifies the identifier of the hierarchy control
	lParam       --> Specifies handle of hierarchy control in the low word,
						  and specifies HCN_BITMAPDBLCLK in the high word.

--------------------------------------------------------------------------------

HCN_ERRSPACE		-Sent when the hiearchy is unable to allocate enough space
						 for the requested command

	wParam       --> Specifies the identifier of the hierarchy control
	lParam       --> Specifies handle of hierarchy control in the low word,
						  and specifies HCN_ERRSPACE in the high word.

--------------------------------------------------------------------------------
*/

#endif
