/* Title: Windows Test Program
	Name: Eric Walsh
	Description  : Test platform for custom controls
*/

#include <stdio.h>
#include <string.h>
#include <windows.h>

#include "control.h"

#include "dragdrop.h"
#include "button.h"
#include "ed_lb.h"
#include "drive.h"
#include "progbar.h"
#include "hrchy.h"

long FAR PASCAL WndProc (HWND, UINT, UINT, LONG);
BOOL CALLBACK AboutDlgProc(HWND, WORD, WORD, LONG);

int DriveButtonInit(HINSTANCE, HINSTANCE);

HBITMAP tclock, diskcrs, sort;
HBITMAP cfolder;                                // Handles to the open and
HBITMAP ofolder;                                // close bitmaps
HCURSOR curs, not;

#define IDD_EDITABLELISTBOX 0x1000
#define IDD_HIERARCHY1      0x1010
#define IDD_HIERARCHY2      0x1020
#define IDD_PROGRESSBAR     0x1030
#define IDD_DRIVEBAR		    0x1040
#define IDD_EDITBUTTON	    0x1050
#define IDD_PROGRESSBUTTON  0x1060

#define IDM_EXIT	102
#define IDM_ABOUT	101

int CreateEditableListBox(HWND);
int CreateHierarchy1(HWND);
int CreateHierarchy2(HWND);
int CreateProgressBar(HWND);
int SetHrchyBitmaps(HWND, int);

int PASCAL WinMain (HANDLE hInstance,   HANDLE hPrevInstance,
			 LPSTR  lpszCmdLine, int nCmdShow)
{
	MSG		msg;
	WNDCLASS	wndclass;
	HWND    	hWndMain;

	if (!hPrevInstance)
	{
		wndclass.style		= CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc	= WndProc;
		wndclass.cbClsExtra	= 0;
		wndclass.cbWndExtra	= 0;
		wndclass.hInstance	= hInstance;
		wndclass.hIcon       = LoadIcon (hInstance, "Wintest" );
		wndclass.hCursor		= LoadCursor (NULL, IDC_ARROW);
		wndclass.hbrBackground	= GetStockObject (LTGRAY_BRUSH);
		wndclass.lpszMenuName	= "WintestMenu";
		wndclass.lpszClassName    = "MainWin";

		RegisterClass (&wndclass);
	}

	HierachyWindowInit(hInstance, hPrevInstance);
	DnDButtonInit(hInstance, hPrevInstance);
	DriveButtonInit(hInstance, hPrevInstance);
	EditListboxInit(hInstance, hPrevInstance);
	ProgressBarInit(hInstance, hPrevInstance);

	hWndMain = CreateWindow ("MainWin", "Windows Custom Control Test Platform",
			WS_OVERLAPPEDWINDOW, 20, 20, 500, 430,
			NULL, NULL, hInstance, NULL );

	ShowWindow (hWndMain, nCmdShow );
	UpdateWindow (hWndMain);

	while (GetMessage (&msg, NULL, 0, 0))
	{
		if (hWndMain == 0 || !IsDialogMessage (hWndMain, &msg))
		{
			TranslateMessage (&msg);
			DispatchMessage (&msg);
		}
	}

	return (msg.wParam);
}

long _export FAR PASCAL WndProc (HWND hWnd, UINT message, UINT wParam, LONG lParam )

{
	static  HWND HrchyWnd, HrchyWnd1;
	static  HWND ButtonhWnd, DrivehWnd;
	static  HWND EdlbhWnd, ProgBarhWnd;
	static  HWND ButtonhWnd1;
	RECT    WinRect;
	char    buffer[256];
	int     cnt;
	static  int state, state2;
	LONG    index, tmp;
	EDLB_COMPAREITEMSTRUCT *ci;
	static  HWND dropWnd;
	FARPROC lpfnDlgProc;

	switch (message)
	{
		case WM_CREATE:		    
			curs =LoadCursor(GetWindowWord(hWnd, GWW_HINSTANCE), "ValCursor");
			not =LoadCursor(GetWindowWord(hWnd, GWW_HINSTANCE), "NotCursor");
			cfolder =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "InactiveFolder");
			ofolder =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "ClosedFolder");

			GetClientRect(hWnd, &WinRect);
			DrivehWnd =CreateWindowEx(WS_EX_ACCEPTFILES, "DriveWindow", "", WS_CHILD | WS_VISIBLE | WS_BORDER, // DRS_MULTIPLESEL |
					0, 0, (WinRect.right *0.7), (WinRect.bottom *0.25),
			hWnd, IDD_DRIVEBAR, GetWindowWord(hWnd, GWW_HINSTANCE), NULL );

			EdlbhWnd =CreateEditableListBox(hWnd);

			HrchyWnd =CreateHierarchy1(hWnd);
			HrchyWnd1 =CreateHierarchy2(hWnd);

			ProgBarhWnd =CreateProgressBar(hWnd);

			ButtonhWnd =CreateWindowEx (WS_EX_ACCEPTFILES, "DnDButton", "<&Go", WS_CHILD | WS_VISIBLE | WS_BORDER,
				   10, 10, 63, 39, hWnd, IDD_EDITBUTTON, GetWindowWord(hWnd, GWW_HINSTANCE), NULL );

			ButtonhWnd1 =CreateWindowEx (WS_EX_ACCEPTFILES, "DnDButton", "&Go>", WS_CHILD | WS_VISIBLE | WS_BORDER,
				 10, 10, 61, 39, hWnd, IDD_PROGRESSBUTTON, GetWindowWord(hWnd, GWW_HINSTANCE), NULL );

			GetEditListboxVersion();
			GetHierarchyVersion();
			GetDNDButtonVersion();
            GetProgressBarVersion();
			break;

		case WM_EDLBCOMPAREITEM:
			ci =(EDLB_COMPAREITEMSTRUCT *)lParam;
			return strcmp((char *)ci->celldata1, (char  *)ci->celldata2);

		case WM_COMMAND:
		{
			switch (wParam)
			{
				case IDD_DRIVEBAR:   // Drive Bank
					switch (HIWORD(lParam))
					{
						case DRN_DRIVECHANGE:
							index =SendMessage(DrivehWnd, DR_GETACTIVEDRIVE, 0, 0L);
							SendMessage(HrchyWnd, HC_SELECTDRIVE, (WORD)index, 0L);
							SetHrchyBitmaps(HrchyWnd, 0);
							break;
					}
					break;

				case IDD_HIERARCHY1:
					switch (HIWORD(lParam))
					{
						case HCN_BITMAPDBLCLK:
						case HCN_DBLCLK:
							cnt =(int)SendMessage(LOWORD(lParam), HC_GETCURSEL, 0, 0L);
							SetHrchyBitmaps(HrchyWnd, cnt);
							break;

						case DDN_DRAGNDROPNEWWIN:
							dropWnd =(HWND)SendMessage(LOWORD(lParam), DD_DRAGNDROPWIN, 0, 0L);
							break;

						case DDN_DONEDRAGNDROP:                             // Move string from one tree to other
							index =SendMessage(dropWnd, HC_GETMOUSEDROPINDEX, 0, 0L);
							tmp =SendMessage(LOWORD(lParam), HC_GETCURSEL, 0, 0L);

							SendMessage(LOWORD(lParam), HC_GETTEXT, (WORD)tmp, (LONG)(LPSTR)buffer);
							SendMessage(dropWnd, HC_ADDSTRING, (WORD)index, (LONG)buffer);
							break;
					}
					break;

				case IDD_EDITABLELISTBOX:
					switch (HIWORD(lParam))
					{
						case ELN_SELCHANGE:
							cnt =(int)SendMessage(LOWORD(lParam), EL_GETSELCOUNT, 0, 0L);
							break;
					}
					break;

				case IDD_EDITBUTTON:
					if (state2 ==0)
					{
						SendMessage(EdlbhWnd, EL_MOVECOLUMN, TRUE, MAKELONG(0, 2));
						SendMessage(EdlbhWnd, EL_RESIZECOLUMN, TRUE, MAKELONG(0, 50));
						state2 =1;
					}
					else
					if (state2 ==1)
					{
						SendMessage(EdlbhWnd, EL_SHOWCOLUMN, 1, 2);
						state2 =2;
					}
					else
					{
						SendMessage(EdlbhWnd, EL_SHOWCOLUMN, 0, 2);
						state2 =0;
					}
					break;

				case IDD_PROGRESSBUTTON:
					if (state ==0)
					{
						SendMessage(ProgBarhWnd, PB_SETBARTEXTCOLOR, 0, RGB(255, 0, 0));
						SetWindowLong(ProgBarhWnd, GWL_STYLE, WS_CHILD | WS_VISIBLE | WS_BORDER | PBS_SHOWTEXTPERCENT | PBS_VERTICALBAR | PBS_3DBAR);
						for (cnt =0; cnt <= 100; cnt++)
							SendMessage(ProgBarhWnd, PB_SETDRAWRANGE, -1, cnt);
						state =1;
						SetWindowText(ButtonhWnd, "&Again");
					}
					else
					if (state ==1)
					{
						SendMessage(ProgBarhWnd, PB_SETBARTEXTCOLOR, 0, RGB(0, 255, 0));
						SetWindowLong(ProgBarhWnd, GWL_STYLE, WS_CHILD | WS_VISIBLE | WS_BORDER | PBS_SHOWTEXTPERCENT | PBS_VERTICALBAR | PBS_BITMAPBAR);
						for (cnt =0; cnt <= 20; cnt++)
							SendMessage(ProgBarhWnd, PB_SETDRAWRANGE, -1, cnt *5);
						state =2;
						SetWindowText(ButtonhWnd, "&Again");
					}
					else
					if (state ==2)
					{
						SendMessage(ProgBarhWnd, PB_SETBARTEXTCOLOR, 0, RGB(0, 0, 255));
						SetWindowLong(ProgBarhWnd, GWL_STYLE, WS_CHILD | WS_VISIBLE | WS_BORDER | PBS_SHOWTEXTPERCENT | PBS_VERTICALBAR);
						for (cnt =0; cnt <= 50; cnt++)
							SendMessage(ProgBarhWnd, PB_SETDRAWRANGE, -1, cnt);
						for (cnt =50; cnt >= 33; cnt--)
							SendMessage(ProgBarhWnd, PB_SETDRAWRANGE, 0, cnt);
						for (cnt =33; cnt <= 75; cnt++)
							SendMessage(ProgBarhWnd, PB_SETDRAWRANGE, 0, cnt);
						for (cnt =75; cnt >= 0; cnt--)
							SendMessage(ProgBarhWnd, PB_SETDRAWRANGE, 0, cnt);
						for (cnt =0; cnt <= 100; cnt++)
							SendMessage(ProgBarhWnd, PB_SETDRAWRANGE, -1, cnt);
						state =0;
						SetWindowText(ButtonhWnd, "&Go");
					}
					break;

				case IDM_ABOUT:
					lpfnDlgProc = MakeProcInstance(AboutDlgProc, GetWindowWord(hWnd, GWW_HINSTANCE));
					DialogBox (GetWindowWord(hWnd, GWW_HINSTANCE), "About", hWnd, lpfnDlgProc );
					FreeProcInstance(lpfnDlgProc);
					break;

				case IDM_EXIT:
					DestroyWindow(hWnd);
					break;
			}
		}
		break;

		case WM_CTLCOLOR:
			switch (HIWORD(lParam))
			{
				case 7:
					return FALSE;
			}
			break;

		case WM_SIZE:
			GetClientRect(hWnd, &WinRect);
			MoveWindow(DrivehWnd, 0, 0, (WinRect.right), 36, 1);
			MoveWindow(EdlbhWnd, 5, (WinRect.bottom * 0.51), (WinRect.right *0.85), (WinRect.bottom *0.45), 1);
			MoveWindow(HrchyWnd, 5, 38, (WinRect.right *0.35), (WinRect.bottom *0.40), 1);
			MoveWindow(HrchyWnd1, (WinRect.right *0.40), 38, (WinRect.right *0.30), (WinRect.bottom *0.40), 1);
			MoveWindow(ProgBarhWnd, (WinRect.right *0.87), (WinRect.bottom *0.12), (WinRect.right *0.11), (WinRect.bottom *0.70), 1);
			MoveWindow(ButtonhWnd, (WinRect.right *0.87), (WinRect.bottom *0.83), (WinRect.right *0.11), (WinRect.bottom *0.13), 1);
			MoveWindow(ButtonhWnd1, (WinRect.right *0.71), 38, (WinRect.right *0.11), (WinRect.bottom *0.13), 1);
			break;

		case WM_DESTROY :
			DestroyCursor(curs);
			DestroyCursor(not);
			DeleteObject(tclock);
			DeleteObject(sort);
			DeleteObject(diskcrs);
			PostQuitMessage (0);
			break;

		default:

		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return FALSE;
}

CreateEditableListBox(HWND hWnd)

{
	HWND    EdlbhWnd;
	CONTROL con[4];
	LONG    sinfo;
	char    buffer[256];
	int     cnt, index;
	EDLB_MSG msg;

	EdlbhWnd =CreateWindow ("EditListBox", "",
				 WS_CHILD | WS_VISIBLE | WS_BORDER | ELS_RESIZECOLUMNS | ELS_MOVEABLECOLUMNS | ELS_MULTICOLUMN | ELS_MULTIPLESEL | ELS_ROWADDRESSABLE | ELS_DRAGDROPSERVER | ELS_SORT | ELS_NUMERICROWLABEL,
				 10, 100, 100, 75,
				 hWnd, IDD_EDITABLELISTBOX, GetWindowWord(hWnd, GWW_HINSTANCE), NULL );

	con[0].width  =115;
	con[0].flags  =ECS_COLUMNHEADING | ECS_SORT | ECS_NORESIZECOLUMN;
	con[0].column_id =0;
	con[0].heading ="Last Name";
	con[0].hflags =0;
	con[0].datatype =ECS_STRING;
	con[1].width  =135;
	con[1].flags  =ECS_COLUMNHEADING | ECS_DESCENDSORT;
	con[1].column_id =1;
	con[1].heading ="First Name";
	con[1].hflags =ECS_CENTER;
	con[1].datatype =ECS_UNSIGNEDNUMERIC;
	con[2].width  =55;
	con[2].flags  =ECS_COLUMNHEADING | ECS_CENTER | ECS_SORT;
	con[2].column_id =2;
	con[2].heading ="M.I.";
	con[2].hflags =ECS_CENTER;
	con[2].datatype =ECS_DATE;
	con[3].width  =105;
	con[3].flags  =ECS_COLUMNHEADING | ECS_RIGHT | ECS_SORT;
	con[3].column_id =3;
	con[3].heading ="S.S. Number";
	con[3].hflags =ECS_RIGHT;
	con[3].datatype =ECS_TIME;
	SendMessage(EdlbhWnd, EL_CONTROLSTRUCT, 4, (LONG)&con);
        
	diskcrs =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "DiskCrs");
	sort =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "Sort");
	tclock =LoadBitmap(GetWindowWord(hWnd, GWW_HINSTANCE), "Clock");
	SendMessage(EdlbhWnd, EL_LOCKCOLUMNS, 1, 0L);
	SendMessage(EdlbhWnd, WM_SETREDRAW, 0, 0L);

	sinfo =MAKELONG(1, 1);
	SendMessage(EdlbhWnd, EL_SORTCOLUMNORDER, 0, sinfo);
	sinfo =MAKELONG(0, 2);
	SendMessage(EdlbhWnd, EL_SORTCOLUMNORDER, 0, sinfo);

	SendMessage(EdlbhWnd, EL_SETDEFAULTTRANSCOLOR, 0, RGB(192, 192, 192));
	
	for (cnt =0; cnt <20; cnt++)
	{
		sprintf(buffer, "col%d,0%c%ld%c%ld%c%ld", cnt, COLUMNSEP, (long)cnt *12345, COLUMNSEP, 19940801L +cnt, COLUMNSEP, 123440L +cnt);
		index =(int)SendMessage(EdlbhWnd, EL_ADDSTRING, -1, (LONG)(LPSTR)buffer);
		msg.col_id =0;
		msg.row_num =index;
		msg.data =diskcrs;
		SendMessage(EdlbhWnd, EL_ADDTRANSPARENTBITMAP, 0, (LONG)(LPSTR)&msg);
		msg.col_id =3;
		msg.row_num =index;
		msg.data =tclock;
		SendMessage(EdlbhWnd, EL_ADDTRANSPARENTBITMAP, 0, (LONG)(LPSTR)&msg);
		msg.col_id =1;
		msg.row_num =index;
		msg.data =sort;
		SendMessage(EdlbhWnd, EL_ADDCELLBITMAP, 0, (LONG)(LPSTR)&msg);
	}

	SendMessage(EdlbhWnd, WM_SETREDRAW, 1, 0L);
	SendMessage(EdlbhWnd, DD_VALIDDROPCURSOR, curs, 0L);
	SendMessage(EdlbhWnd, DD_INVALIDDROPCURSOR, not, 0L);
	SendMessage(EdlbhWnd, EL_GETTEXT, 0, (LONG)buffer);
	sprintf(buffer, "Insert%c1200%c19940301%c123456", COLUMNSEP, COLUMNSEP, COLUMNSEP);
	SendMessage(EdlbhWnd, EL_INSERTSTRING, 1, (LONG)(LPSTR)buffer);

	msg.col_id =2;
	msg.row_num =2;
	msg.data =19940911L;
	SendMessage(EdlbhWnd, EL_SETCELLDATA, 0, (LONG)&msg);

	return EdlbhWnd;
}

CreateProgressBar(HWND hWnd)
{
	HWND ProgBarhWnd;

	ProgBarhWnd =CreateWindow("ProgressBar", "", WS_CHILD | WS_VISIBLE | WS_BORDER | PBS_SHOWTEXTPERCENT | PBS_BITMAPBAR | PBS_VERTICALBAR,
			0,0,10,10, hWnd, IDD_PROGRESSBAR, GetWindowWord(hWnd, GWW_HINSTANCE), NULL );

	SendMessage(ProgBarhWnd, PB_SETPROGBARBITMAP, ofolder, 0L); // from hiearchy

	return ProgBarhWnd;
}

CreateHierarchy1(HWND hWnd)
{
	HWND HrchyWnd;

	HrchyWnd =CreateWindowEx(WS_EX_ACCEPTFILES, "Hierarchy", "", WS_CHILD | WS_VISIBLE | WS_BORDER | HCS_DRAGDROPSERVER |  HCS_SORT | HCS_DIRECTORYTREE,
			0,0,10,10, hWnd, IDD_HIERARCHY1, GetWindowWord(hWnd, GWW_HINSTANCE), NULL );

	SendMessage(HrchyWnd, HC_SELECTDRIVE, 3, 0);
	SetHrchyBitmaps(HrchyWnd, 0);
	SendMessage(HrchyWnd, DD_VALIDDROPCURSOR, curs, 0L);
	SendMessage(HrchyWnd, DD_INVALIDDROPCURSOR, not, 0L);

	return HrchyWnd;
}

CreateHierarchy2(HWND hWnd)
{
	HWND HrchyWnd1;
	int  index;

	HrchyWnd1 =CreateWindowEx(WS_EX_ACCEPTFILES, "Hierarchy", "", WS_CHILD | WS_VISIBLE | WS_BORDER  | HCS_SORT,
			0,0,10,10, hWnd, IDD_HIERARCHY2, GetWindowWord(hWnd, GWW_HINSTANCE), NULL );

	SendMessage(HrchyWnd1, HC_SETBITMAPTRANSCOLOR, 0, RGB(255, 0, 0));   // Sets color to make transparent for bitmap
	SendMessage(HrchyWnd1, HC_ADDSTRING, -1, (LONG)"Fldr One");
	SendMessage(HrchyWnd1, HC_SETITEMBITMAP, 0, ofolder);
	SendMessage(HrchyWnd1, HC_ADDSTRING, 0, (LONG)"Sub Fldr ONE-A");
	SendMessage(HrchyWnd1, HC_ADDSTRING, 0, (LONG)"Sub Fldr TWO-A");
	SendMessage(HrchyWnd1, HC_ADDSTRING, 1, (LONG)"Sub Fldr ONE-B");

	index =(int)SendMessage(HrchyWnd1, HC_ADDSTRING, -1, (LONG)"Fldr Two");
	SendMessage(HrchyWnd1, HC_SETITEMBITMAP, index, ofolder);
	SendMessage(HrchyWnd1, HC_ADDSTRING, index, (LONG)"Sub Fldr TWO-A");
	SendMessage(HrchyWnd1, HC_ADDSTRING, index, (LONG)"Sub Fldr TWO-B");
	SendMessage(HrchyWnd1, HC_ADDSTRING, index, (LONG)"Sub Fldr TWO-C");
	index =(int)SendMessage(HrchyWnd1, HC_ADDSTRING, index, (LONG)"Sub Fldr TWO-D");
	SendMessage(HrchyWnd1, HC_ADDSTRING, index, (LONG)"Sub Fldr TWO-D1");

	SendMessage(HrchyWnd1, DD_VALIDDROPCURSOR, curs, 0L);
	SendMessage(HrchyWnd1, DD_INVALIDDROPCURSOR, not, 0L);
	return HrchyWnd1;
}

SetHrchyBitmaps(HWND HrchyWnd, int index)
{
	int cnt, count;

	count =(int)SendMessage(HrchyWnd, HC_SETBITMAPTRANSCOLOR, 0, RGB(255,0,0));
	if (SendMessage(HrchyWnd, HC_GETITEMEXPANDEDSTATE, index, 0L) ==1)  // Item is expanded
	{
		count =(int)SendMessage(HrchyWnd, HC_GETCOUNT, index, TRUE);
		if ((SendMessage(HrchyWnd, HC_GETITEMBITMAP, index, ofolder)) ==NULL)
			SendMessage(HrchyWnd, HC_SETITEMBITMAP, index, ofolder);
		for (cnt =0; cnt <count; cnt++)
		{
			if ((SendMessage(HrchyWnd, HC_GETITEMBITMAP, index +cnt +1, ofolder)) ==NULL)
				SendMessage(HrchyWnd, HC_SETITEMBITMAP, index +cnt +1, cfolder);
		}

	}
	else
		SendMessage(HrchyWnd, HC_SETITEMBITMAP, cnt, cfolder);

	return TRUE;	
}

BOOL _export CALLBACK AboutDlgProc(HWND hWnd, WORD message, WORD wParam, LONG lParam)

{
   switch (message)
   {
      case WM_COMMAND:
	 switch (wParam)
	 {
	    case IDOK:
	    case IDCANCEL:
               EndDialog(hWnd, 0);
	       break;
	 }
	 break;
   }

   return FALSE;
}

